""" A QueueTableCommand Command """


from cleo import Command

from masonite.helpers.filesystem import copy_migration


class QueueTableCommand(Command):
    """
    Create migration files for the queue feature

    queue:table
    """

    def handle(self):
        copy_migration('masonite/snippets/migrations/create_failed_jobs_table.py')
        self.info('Migration created successfully')



from .static import static
from .password import password
from .validator import validate
from .misc import random_string, dot, clean_request_input, HasColoredCommands, Compact as compact
from .Extendable import Extendable
from .time import cookie_expire_time
from .optional import Optional as optional
from .structures import config, Dot
from .migrations import has_unmigrated_migrations



"""A Module For Manipulating Code Structures."""

import pydoc


class Dot:

    def dot(self, search, dictionary):

        if '.' not in search:
            return dictionary[search]

        searching = search.split('.')
        while len(searching) > 0:
            dic = dictionary
            for search in searching:
                dic = dic.get(search)

            if not isinstance(dic, dict):
                return dic

            del searching[-1]

    def locate(self, search_path, default=''):
        """Locate the object from the given search path

        Arguments:
            search_path {string} -- A search path to fetch the object from like config.application.debug.

        Keyword Arguments:
            default {string} -- A default string if the search path is not found (default: {''})

        Returns:
            any -- Could be a string, object or anything else that is fetched.
        """
        value = self.find(search_path, default)

        if isinstance(value, dict):
            return self.dict_dot('.'.join(search_path.split('.')[3:]), value)

        if value is not None:
            return value

    def dict_dot(self, search, dictionary):
        """Takes a dot notation representation of a dictionary and fetches it from the dictionary.

        This will take something like s3.locations and look into the s3 dictionary and fetch the locations
        key.

        Arguments:
            search {string} -- The string to search for in the dictionary using dot notation.
            dictionary {dict} -- The dictionary to search through.

        Returns:
            string -- The value of the dictionary element.
        """
        if "." in search:
            key, rest = search.split(".", 1)
            try:
                return self.dict_dot(dictionary[key], rest)
            except (KeyError, TypeError):
                pass
        else:
            try:
                return dictionary[search]
            except TypeError:
                pass

        return self.dict_dot(dictionary, search)

    def find(self, search_path, default=''):
        """Used for finding both the uppercase and specified version.

        Arguments:
            search_path {string} -- The search path to find the module, dictionary key, object etc.
                                    This is typically in the form of dot notation 'config.application.debug'

        Keyword Arguments:
            default {string} -- The default value to return if the search path could not be found. (default: {''})

        Returns:
            any -- Could be a string, object or anything else that is fetched.
        """
        value = pydoc.locate(search_path)

        if value:
            return value

        paths = search_path.split('.')

        value = pydoc.locate('.'.join(paths[:-1]) + '.' + paths[-1].upper())

        if value:
            return value

        search_path = -1

        # Go backwards through the dot notation until a match is found.
        while search_path < len(paths):
            value = pydoc.locate('.'.join(paths[:search_path]) + '.' + paths[search_path].upper())

            if value:
                break

            value = pydoc.locate('.'.join(paths[:search_path]) + '.' + paths[search_path])

            if value:
                break

            if default:
                return default

            search_path -= 1

        return value


def config(path, default=''):
    return Dot().locate('config.' + path, default)



import subprocess
from masonite.helpers import config


def has_unmigrated_migrations():
    if not config('application.debug'):
        return False

    from wsgi import container
    from config.database import DB
    try:
        DB.connection()
    except Exception:
        return False

    migration_directory = ['databases/migrations']
    for key, value in container.providers.items():
        if type(key) == str and 'MigrationDirectory' in key:
            migration_directory.append(value)

    for directory in migration_directory:
        try:
            output = bytes(subprocess.check_output(
                ['orator', 'migrate:status', '-c',
                    'config/database.py', '-p', directory]
            )).decode('utf-8')

            if 'No' in output:
                return True
        except Exception:
            pass

    return False
